def get_citation():
    BIB = """
@article{Borry2021_pydamage,
    author = {Borry, Maxime and Huebner, Alexander and Rohrlach, Adam B and Warinner, Christina G},
    doi = {10.1101/2021.03.24.436838},
    elocation-id = {2021.03.24.436838},
    eprint = {https://www.biorxiv.org/content/early/2021/03/24/2021.03.24.436838.full.pdf},
    journal = {bioRxiv},
    publisher = {Cold Spring Harbor Laboratory},
    title = {PyDamage: automated ancient damage identification and estimation for contigs in ancient DNA de novo assembly},
    url = {https://www.biorxiv.org/content/early/2021/03/24/2021.03.24.436838},
    year = {2021}
}
"""
    print(BIB)